import type { TableData, BaseFieldInfo } from '@/types/Tables/table'
import type { BaseTableInfo } from '@/types/Tables/tablePageData'
import type { ResponseInfo } from '@/types/axios'
import type {
  BaseFieldItem,
  TableFields,
  TableInfoItem,
  SaveFields,
  SaveCustomIncicatorScheme,
  CustomIndicatorScheme,
} from '@/types/Tables/table'
import type { Ref } from 'vue'
import type { PaginationConfig } from '@/types/Tables/pagination'

import { resetReactive } from '@/utils/common'
import axiosInstance from '@/utils/axios/axiosInstance'

export function useTable() {
  const getData = async (
    url: string,
    config: Object = {},
  ): Promise<Array<TableData>> => {
    try {
      const res = (await axiosInstance.get(url, config)) as ResponseInfo

      if (res.code !== 0) throw new Error('请求失败')
      return res.data as TableData[]
    } catch (err) {
      console.log('数据请求失败')
      console.log(err)
      return []
    }
  }

  /**
   * @description: 判断当前字段是否是需要固定的字段
   * @param {*} info 字段信息
   * @return {boolean}
   */
  const isFixedField = (
    fixedFields: Array<string>,
    info: BaseFieldInfo,
  ): boolean => {
    return fixedFields.includes(info.name)
  }

  /**
   * @description: 生成一个唯一的方案键
   * @param {string} schemeName 方案名
   * @param {string} return 唯一键
   * @return {*}
   */
  function generateUniqueKey(schemeName: string): string {
    const hash = Array.from(schemeName).reduce((acc, char) => {
      return (acc * 31 + char.charCodeAt(0)) % 1e12
    }, 0)

    return String(hash).padStart(12, '0')
  }

  /**
   * @description: 获取保存的表格信息
   * @param {string} name 这个是整个表格的名字，用于区分不同表格
   * @param {string} customSchemeName 这个是自定义方案的名称
   * @return {SaveCustomIncicatorScheme | SaveFields | undefined} 返回的可能是整个表格的所有方案，也可能只是某个具体的方案，取决于传参
   */
  function getTableInfo(name: string): SaveCustomIncicatorScheme | undefined
  function getTableInfo(
    name: string,
    customSchemeName: string,
  ): SaveFields | undefined

  function getTableInfo(
    name: string,
    customSchemeName?: string,
  ): SaveCustomIncicatorScheme | SaveFields | undefined {
    try {
      let info = localStorage.getItem(name)
      if (!info) throw new Error('没有该信息')
      let keyName: string = ''
      if (customSchemeName) {
        keyName = generateUniqueKey(customSchemeName)
        return (JSON.parse(info) as SaveCustomIncicatorScheme)[keyName]
      }
      return JSON.parse(info) as SaveCustomIncicatorScheme
    } catch (err) {
      console.log('获取失败')
      console.log(err)
      return undefined
    }
  }

  /**
   * @description: 保存表格信息
   * @return {*}
   */
  const saveTableInfo = (
    name: string,
    tableInfo: TableInfoItem<TableData, BaseFieldItem<TableFields>>,
    isSaveToTemplate: boolean, // 是否要保存成自定义方案
    customSchemeName?: string, // 自定义指标的方案名
  ) => {
    try {
      customSchemeName = customSchemeName ? customSchemeName : '默认'
      let keyName = generateUniqueKey(customSchemeName)

      let result: SaveFields = {
        name: keyName, // 这个是拿来作为标识符，也同时是这个方案的键
        label: customSchemeName,
        fields: tableInfo.fields, // 字段数组
        tableSortedFields: tableInfo.tableSortedFields, // 排序过后的表格字段，仅用于展示表格字段
      }

      let saveInfo = getTableInfo(name)
      // 没有这个信息需要先去创建一个默认版本
      if (!saveInfo) {
        saveInfo = {
          [keyName]: result,
        }
      }
      saveInfo[keyName] = result
      // if (isSaveToTemplate) {
      //   saveInfo[keyName] = result
      // } else {
      //   saveInfo['default'] = result
      // }

      localStorage.setItem(name, JSON.stringify(saveInfo))
    } catch (err) {
      console.log('保存失败')
      console.log(err)
    }
  }

  /**
   * @description: 获取所有自定义方案的数组
   * @param {string} name 表格名
   * @return {*}
   */
  const getCustomScheme = (name: string): Array<CustomIndicatorScheme> => {
    let info = getTableInfo(name)
    let result: Array<CustomIndicatorScheme> = []
    if (!info) return []
    for (let [key, scheme] of Object.entries(info)) {
      let name = decodeURIComponent(key)
      let label = decodeURIComponent(scheme.label)

      result.push({
        name,
        label,
      })
    }
    return result
  }

  /**
   * @description: 更新自定义指标和表格字段
   * @param {*} newIndicator  新的指标信息
   * @param {*} newSortedTablefileds 新的表格字段信息
   * @return {*}
   */
  const updateTableFields = (
    tableInfo: BaseTableInfo,
    activeName: string,
    newTableFiles: Array<BaseFieldItem<TableFields>>,
    newSortedTablefileds: Array<TableFields>,
  ): Promise<boolean> => {
    return new Promise((reslove, reject) => {
      try {
        let oldTablefields = tableInfo[activeName].fields
        let oldSortedFields = tableInfo[activeName].tableSortedFields

        oldTablefields.splice(0, oldTablefields.length, ...newTableFiles)

        oldSortedFields.splice(
          0,
          oldSortedFields.length,
          ...newSortedTablefileds,
        )
        reslove(true)
      } catch (err) {
        console.log('更新失败')
        console.log(err)
        reject(false)
      }
    })
  }

  /**
   * @description: 初始化自定义指标的字段信息
   * @return {*}
   */
  const initIndicatorFields = (
    indicatorFields: Array<BaseFieldItem<TableFields>>,
    propsTableFields: Array<BaseFieldItem<TableFields>>,
    defaultActiveNav: Ref<string>,
  ) => {
    resetReactive(indicatorFields)
    indicatorFields.splice(
      0,
      indicatorFields.length,
      ...JSON.parse(JSON.stringify(propsTableFields)),
    )
    defaultActiveNav.value = indicatorFields[0].name
  }

  /**
   * @description: 初始化表格字段，把所有字段都放到一个数组中
   * @return {*}
   */
  const initTableFields = (
    tableFieldsInfo: Array<TableFields>,
    sortedTableFields: Array<TableFields>,
    indicatorFields: Array<BaseFieldItem<TableFields>>,
  ) => {
    resetReactive(tableFieldsInfo)
    // 如果传入了排序字段，则直接使用排序字段，说明已经有缓存了
    if (sortedTableFields.length > 0) {
      tableFieldsInfo.splice(0, tableFieldsInfo.length, ...sortedTableFields)
    } else {
      // 没传入排序字段，则根据固定字段和自定义指标字段进行排序
      indicatorFields.forEach(item => {
        item.children.forEach(child => {
          if (child.fixed || child.state) {
            item.value.push(child.name)
            child.state = true
          }
          tableFieldsInfo.push(JSON.parse(JSON.stringify(child)))
        })
      })
    }
  }

  /**
   * @description: 缓存表格数据。只有是远程查询的时候才会开启缓存
   * @param {*} newData 新的表格数据
   * @return {*}
   */
  const setCacheTableData = (
    isRemote: boolean,
    paginationConfig: PaginationConfig,
    cacheTableData: Array<Array<TableData>>,
    newData: Array<TableData>,
  ) => {
    if (isRemote) {
      // 这页没有数据，就缓存一下
      let curPage = paginationConfig.curPage
      let curPageSize = paginationConfig.curPageSize
      if (!cacheTableData[curPage] || cacheTableData[curPage].length === 0) {
        cacheTableData[curPage] = newData.splice(0, curPageSize) // 缓存数据，后面的splice是防止返回的数据超出当前页长度
      }
    }
  }

  return {
    getData,
    isFixedField,
    saveTableInfo,
    getTableInfo,
    updateTableFields,
    initTableFields,
    initIndicatorFields,
    setCacheTableData,
    getCustomScheme,
  }
}
